﻿using System;
using Ozeki.Media;
using Ozeki.MediaGateway;
using Ozeki.VoIP;

namespace MediaGatewayClients.MediaGateway
{
    class MediaGatewayClient : IExtension
    {
        IPBXCallFactory pbxCallFactory;
        IStreamService streamService;
        ICall call;
        ICallManager callManager;

        MediaConnector mediaConnector;
        PhoneCallAudioSender phoneCallAudioSender;
        PhoneCallAudioReceiver phoneCallAudioReceiver;

        MediaGatewayAudioReceiver mediaGatewayAudioReceiver;
        MediaGatewayAudioSender mediaGatewayAudioSender;

        string publishStreamName;
        string playStreamName;

        IClient client;

        public MediaGatewayClient(string id, IClient client, IPBXCallFactory pbxCallFactory, IStreamService streamService, ICallManager callManager)
        {
            ExtensionID = id;
            this.pbxCallFactory = pbxCallFactory;
            this.streamService = streamService;
            this.client = client;
            this.callManager = callManager;

            mediaConnector = new MediaConnector();
            phoneCallAudioSender = new PhoneCallAudioSender();
            phoneCallAudioReceiver = new PhoneCallAudioReceiver();
        }

        #region IExtension

        public string ExtensionID { get; private set; }

        public string Type
        {
            get { return "MediaGatewayExtension"; }
        }

        public void OnMessageReceived(Message message)
        {
        }

        public void OnCalled(IPBXCall call, IncomingCallParams callParams)
        {
            call.CallStateChanged += Call_StateChanged;
        }

        #endregion

        /// <summary>
        /// Call the given dial number.
        /// </summary>
        /// <param name="dialNumber">dial number</param>
        public virtual void Call(string dialNumber)
        {
            var dialParams = new DialParameters(dialNumber);
            var pbxCall = pbxCallFactory.CreateServerCall(this, dialParams);

            call = pbxCall;
            call.CallStateChanged += Call_StateChanged;

            callManager.RouteCall(pbxCall);
        }

        /// <summary>
        /// Hangup the call.
        /// </summary>
        public virtual void HangUp()
        {
            if (call != null)
            {
                call.HangUp();
                call = null;
            }
        }

        /// <summary>
        /// Accept the incoming call request.
        /// </summary>
        public virtual void Accept()
        {
            if (call != null)
                call.Answer();
        }

        /// <summary>
        /// Reject the incoming call request.
        /// </summary>
        public void Reject()
        {
            if (call != null)
            {
                call.Reject();
                call = null;
            }
        }

        public void StreamPublished(IMediaStream mediaStream)
        {
            mediaGatewayAudioSender = new MediaGatewayAudioSender(mediaStream);
            mediaConnector.Connect(mediaGatewayAudioSender, phoneCallAudioSender);
        }

        public void StreamClosed()
        {
            if (mediaGatewayAudioSender != null)
            {
                mediaConnector.Disconnect(mediaGatewayAudioSender, phoneCallAudioSender);
                mediaGatewayAudioSender.Dispose();
                mediaGatewayAudioSender = null;
            }

        }

        /// <summary>
        /// This will be called when the state of a call has changed.
        /// </summary>
        void Call_StateChanged(object sender, CallStateChangedArgs e)
        {
            if (e.State == CallState.Ringing)
            {
                OnCallRequest(call.DialInfo.CallerID);
            }


            if (e.State == CallState.Answered)
            {
                phoneCallAudioSender.AttachToCall(call);
                phoneCallAudioReceiver.AttachToCall(call);

                publishStreamName = Guid.NewGuid().ToString();
                playStreamName = Guid.NewGuid().ToString();

                var mediaStream = streamService.CreateStream(playStreamName);
                mediaGatewayAudioReceiver = new MediaGatewayAudioReceiver(mediaStream);

                mediaConnector.Connect(phoneCallAudioReceiver, mediaGatewayAudioReceiver);

                OnInCall(publishStreamName, playStreamName);
            }


            if (e.State.IsCallEnded())
            {
                call.CallStateChanged -= Call_StateChanged;
                phoneCallAudioSender.Detach();
                phoneCallAudioReceiver.Detach();

                streamService.RemoveStream(mediaGatewayAudioReceiver.MediaStream);
                mediaConnector.Disconnect(phoneCallAudioReceiver, mediaGatewayAudioReceiver);
                mediaGatewayAudioReceiver.Dispose();

                mediaGatewayAudioReceiver = null;
                call = null;
            }

            OnCallStateChanged(e.State);
        }

        #region Client-side method invoke

        /// <summary>
        /// Invoke client's callback method. Notifies the client that call state has changed.
        /// </summary>
        /// <param name="callState"></param>
        void OnCallStateChanged(CallState callState)
        {
            Console.WriteLine("OnCallStateChanged");
            client.InvokeMethod("OnCallStateChanged", callState);
        }

        /// <summary>
        /// Invokes client's callback method. Notifies the client that it is ready to start or receive a call.
        /// </summary>
        public void OnSetReadyStatus()
        {
            try
            {
                client.InvokeMethod("OnSetReadyStatus", ExtensionID);
            }
            catch (Exception)
            { }
        }

        /// <summary>
        /// Invokes client's callback method. Notifies the client that it received a new call from another extension.
        /// </summary>
        /// <param name="requestOwner">The call request sender's clientId.</param>
        public void OnCallRequest(string requestOwner)
        {
            Console.WriteLine("Call request received from {0} to {1}", requestOwner, "Name");

            client.InvokeMethod("OnCallRequest", requestOwner);
        }

        /// <summary>
        /// Invoke client's callback method. Sends the publish and receive stream ID.
        /// </summary>
        public void OnInCall(string senderPublishID, string receiverPlayID)
        {
            Console.WriteLine("Sends 'start publishing' sign to the clients.");
            client.InvokeMethod("OnInCall", senderPublishID, receiverPlayID);
        }

        #endregion
    }
}
