﻿using System;
using System.Collections.Generic;
using Ozeki.MediaGateway;
using Ozeki.VoIP;

namespace MediaGatewayClients.MediaGateway
{
    class MyMediaGateway : Ozeki.MediaGateway.MediaGateway
    {
        Dictionary<IClient, MediaGatewayClient> myClients;
        IStreamService streamService;
        IPBXCallFactory pbxCallFactory;
        IExtensionContainer extensionContainer;
        ICallManager callManager;
        string localAddress;

        AccountPool accountPool;

        public MyMediaGateway(MediaGatewayConfig mediaGateway, IPBXCallFactory pbxCallFactory, IExtensionContainer extensionContainer, ICallManager callManager, string localAddress):base(mediaGateway)
        {
            Console.WriteLine("MediaGateway starting...");

            myClients = new Dictionary<IClient, MediaGatewayClient>();

            this.pbxCallFactory = pbxCallFactory;
            this.extensionContainer = extensionContainer;
            this.callManager = callManager;
            this.localAddress = localAddress;

            SetAvailableAccounts();
        }

        public override void OnStart()
        {
            Console.WriteLine("MediaGateway started.");

            streamService = GetService<IStreamService>();
        }

        public override void OnClientConnect(IClient client, object[] parameters)
        {
            Console.WriteLine("New client connected. Remote address: {0}", client.RemoteAddress);

            lock (myClients)
            {
                var account = accountPool.GetAccount();

                if (account == null)
                {
                    client.InvokeMethod("OnRegistrationStateChanged", "Connection limit");
                    client.Close();
                    return;
                }

                var mediaGatewayClient = new MediaGatewayClient(account, client, pbxCallFactory, streamService, callManager);
                myClients.Add(client, mediaGatewayClient);

                extensionContainer.TryAddExtension(mediaGatewayClient);

                mediaGatewayClient.OnSetReadyStatus();
            }
        }

        public override void OnClientDisconnect(IClient client)
        {
            Console.WriteLine("Client disconnected. Remote address: {0}", client.RemoteAddress);

            lock (myClients)
            {
                MediaGatewayClient mediaGatewayClient;
                if (myClients.TryGetValue(client, out mediaGatewayClient))
                {
                    myClients.Remove(client);
                    extensionContainer.TryRemoveExtension(mediaGatewayClient);
                }
            }
        }

        /// <summary>
        /// Indicates the client starts publish his IMediastream.
        /// </summary>
        /// <param name="client">That client reference who call this method.</param>
        /// <param name="mediaStream">Published IMediaStream</param>
        public override void OnStreamPublishStart(IClient client, IMediaStream mediaStream)
        {
            Console.WriteLine("Media stream published. Stream name {0}", mediaStream.Name);

            lock (myClients)
            {
                MediaGatewayClient myClient;
                if (myClients.TryGetValue(client, out myClient))
                    myClient.StreamPublished(mediaStream);
            }
        }

        /// <summary>
        /// Indicates the client close his IMediaStream.
        /// </summary>
        /// <param name="client">That client reference who call this method.</param>
        /// <param name="mediaStream">Closed IMediaStream</param>
        public override void OnStreamClose(IClient client, IMediaStream mediaStream)
        {
            Console.WriteLine("Media stream closed.");

            lock (myClients)
            {
                MediaGatewayClient myClient;
                if (myClients.TryGetValue(client, out myClient))
                    myClient.StreamClosed();
            }
        }

        /// <summary>
        /// Forward the call request to the MyClient.
        /// </summary>
        /// <param name="client">That client reference who call this method.</param>
        /// <param name="phoneNumber">dial number</param>
        public void Call(IClient client, string phoneNumber)
        {
            lock (myClients)
            {
                MediaGatewayClient myClient;
                if (myClients.TryGetValue(client, out myClient))
                    myClient.Call(phoneNumber);
            }
        }

        /// <summary>
        /// Forward the hangup request to the MyClient.
        /// </summary>
        /// <param name="client">That client reference who call this method.</param>
        public void HangUp(IClient client)
        {
            lock (myClients)
            {
                MediaGatewayClient myClient;
                if (myClients.TryGetValue(client, out myClient))
                    myClient.HangUp();
            }
        }

        /// <summary>
        /// Forwards the Accept request to the MyClient.
        /// </summary>
        /// <param name="client">That client reference who call this method.</param>
        public void Accept(IClient client)
        {
            lock (myClients)
            {
                MediaGatewayClient myClient;
                if (myClients.TryGetValue(client, out myClient))
                    myClient.Accept();
            }
        }

        /// <summary>
        /// Reject an incoming call request.
        /// </summary>
        /// <param name="client">That client reference who call this method.</param>
        public void Reject(IClient client)
        {
            lock (myClients)
            {
                MediaGatewayClient myClient;
                if (myClients.TryGetValue(client, out  myClient))
                    myClient.Reject();
            }
        }

        private void SetAvailableAccounts()
        {
            accountPool = new AccountPool();

            for (int i = 900; i < 1000; i++)
            {
                var userName = i.ToString();

                accountPool.AddAccount(userName);
            }
        }
    }
}
